@@ -19,10 +19,10 @@ class ClsSourceCifar10(object):
             root=root, train=(split == 'train'), download=False)
         self.labels = self.cifar.targets
 
-    def get_length(self):
+    def __len__(self):
         return len(self.cifar)
 
-    def get_sample(self, idx):
+    def __getitem__(self, idx):
         img = Image.fromarray(self.cifar.data[idx])
         label = self.labels[idx]  # img: HWC, RGB
         result_dict = {'img': img, 'gt_labels': label}
@@ -42,10 +42,10 @@ class ClsSourceCifar100(object):
 
         self.labels = self.cifar.targets
 
-    def get_length(self):
+    def __len__(self):
         return len(self.cifar)
 
-    def get_sample(self, idx):
+    def __getitem__(self, idx):
         img = Image.fromarray(self.cifar.data[idx])
         label = self.labels[idx]  # img: HWC, RGB
         result_dict = {'img': img, 'gt_labels': label}
