@@ -3,6 +3,7 @@ import copy
 import logging
 import math
 import os.path as osp
+import warnings
 
 import cv2
 import mmcv
@@ -13,13 +14,16 @@ from torchvision.transforms import functional as F
 from easycv.datasets.registry import PIPELINES
 from easycv.datasets.shared.pipelines.transforms import Compose
 
+try:
+    from panopticapi.utils import rgb2id
+except ImportError:
+    rgb2id = None
+
 
 @PIPELINES.register_module()
 class MMToTensor:
     """Transform image to Tensor.
-
     Required key: 'img'. Modifies key: 'img'.
-
     Args:
         results (dict): contain all information about training.
     """
@@ -32,9 +36,7 @@ class MMToTensor:
 @PIPELINES.register_module()
 class NormalizeTensor:
     """Normalize the Tensor image (CxHxW), with mean and std.
-
     Required key: 'img'. Modifies key: 'img'.
-
     Args:
         mean (list[float]): Mean values of 3 channels.
         std (list[float]): Std values of 3 channels.
@@ -53,13 +55,10 @@ class NormalizeTensor:
 @PIPELINES.register_module
 class MMMosaic(object):
     """Mosaic augmentation.
-
     Given 4 images, mosaic transform combines them into
     one output image. The output image is composed of the parts from each sub-
     image.
-
     .. code:: text
-
                         mosaic transform
                            center_x
                 +------------------------------+
@@ -76,14 +75,11 @@ class MMMosaic(object):
                 +----|-------------+-----------+
                      |             |
                      +-------------+
-
      The mosaic transform steps are as follows:
-
          1. Choose the mosaic center as the intersections of 4 images
          2. Get the left top image according to the index, and randomly
             sample another 3 images from the custom dataset.
          3. Sub image will be cropped if image is larger than mosaic patch
-
     Args:
         img_scale (Sequence[int]): Image size after mosaic pipeline of single
            image. Default to (640, 640).
@@ -103,10 +99,8 @@ class MMMosaic(object):
 
     def __call__(self, results):
         """Call function to make a mosaic of image.
-
         Args:
             results (dict): Result dict.
-
         Returns:
             dict: Result dict with mosaic transformed.
         """
@@ -116,10 +110,8 @@ class MMMosaic(object):
 
     def get_indexes(self, dataset):
         """Call function to collect indexes.
-
         Args:
             dataset (:obj:`DetImagesMixDataset`): The dataset.
-
         Returns:
             list: indexes.
         """
@@ -128,10 +120,8 @@ class MMMosaic(object):
 
     def _mosaic_transform(self, results):
         """Mosaic transform function.
-
         Args:
             results (dict): Result dict.
-
         Returns:
             dict: Updated result dict.
         """
@@ -215,14 +205,12 @@ class MMMosaic(object):
     def _mosaic_combine(self, loc, center_position_xy, img_shape_wh):
         """Calculate global coordinate of mosaic image and local coordinate of
         cropped sub-image.
-
         Args:
             loc (str): Index for the sub-image, loc in ('top_left',
               'top_right', 'bottom_left', 'bottom_right').
             center_position_xy (Sequence[float]): Mixing center for 4 images,
                 (x, y).
             img_shape_wh (Sequence[int]): Width and height of sub-image
-
         Returns:
             tuple[tuple[float]]: Corresponding coordinate of pasting and
                 cropping
@@ -285,9 +273,7 @@ class MMMosaic(object):
 @PIPELINES.register_module
 class MMMixUp:
     """MixUp data augmentation.
-
     .. code:: text
-
                          mixup transform
                 +------------------------------+
                 | mixup image   |              |
@@ -301,14 +287,11 @@ class MMMixUp:
                 |      |-----------------+     |
                 |             pad              |
                 +------------------------------+
-
      The mixup transform steps are as follows::
-
         1. Another random image is picked by dataset and embedded in
            the top left patch(after padding and resizing)
         2. The target of mixup transform is the weighted average of mixup
            image and origin image.
-
     Args:
         img_scale (Sequence[int]): Image output size after mixup pipeline.
            Default: (640, 640).
@@ -352,10 +335,8 @@ class MMMixUp:
 
     def __call__(self, results):
         """Call function to make a mixup of image.
-
         Args:
             results (dict): Result dict.
-
         Returns:
             dict: Result dict with mixup transformed.
         """
@@ -365,10 +346,8 @@ class MMMixUp:
 
     def get_indexes(self, dataset):
         """Call function to collect indexes.
-
         Args:
             dataset (:obj:`DetImagesMixDataset`): The dataset.
-
         Returns:
             list: indexes.
         """
@@ -383,10 +362,8 @@ class MMMixUp:
 
     def _mixup_transform(self, results):
         """MixUp transform function.
-
         Args:
             results (dict): Result dict.
-
         Returns:
             dict: Updated result dict.
         """
@@ -494,7 +471,6 @@ class MMMixUp:
 
     def _filter_box_candidates(self, bbox1, bbox2):
         """Compute candidate boxes which include following 5 things:
-
         bbox1 before augment, bbox2 after augment, min_bbox_size (pixels),
         min_area_ratio, max_aspect_ratio.
         """
@@ -523,10 +499,8 @@ class MMMixUp:
 @PIPELINES.register_module
 class MMRandomAffine:
     """Random affine transform data augmentation. for yolox
-
     This operation randomly generates affine transform matrix which including
     rotation, translation, shear and scaling transforms.
-
     Args:
         max_rotate_degree (float): Maximum degrees of rotation transform.
             Default: 10.
@@ -727,7 +701,6 @@ class MMPhotoMetricDistortion:
     """Apply photometric distortion to image sequentially, every transformation
     is applied with a probability of 0.5. The position of random contrast is in
     second or second to last.
-
     1. random brightness
     2. random contrast (mode 0)
     3. convert color from BGR to HSV
@@ -736,7 +709,6 @@ class MMPhotoMetricDistortion:
     6. convert color from HSV to BGR
     7. random contrast (mode 1)
     8. randomly swap channels
-
     Args:
         brightness_delta (int): delta of brightness.
         contrast_range (tuple): range of contrast.
@@ -756,10 +728,8 @@ class MMPhotoMetricDistortion:
 
     def __call__(self, results):
         """Call function to perform photometric distortion on images.
-
         Args:
             results (dict): Result dict from loading pipeline.
-
         Returns:
             dict: Result dict with images distorted.
         """
@@ -1284,7 +1254,6 @@ class MMRandomCrop:
             on cropped instance masks. Default False.
         bbox_clip_border (bool, optional): Whether clip the objects outside
             the border of the image. Defaults to True.
-
     Note:
         - If the image is smaller than the absolute crop size, return the
             original image.
@@ -1330,13 +1299,11 @@ class MMRandomCrop:
     def _crop_data(self, results, crop_size, allow_negative_crop):
         """Function to randomly crop images, bounding boxes, masks, semantic
         segmentation maps.
-
         Args:
             results (dict): Result dict from loading pipeline.
             crop_size (tuple): Expected absolute size after cropping, (h, w).
             allow_negative_crop (bool): Whether to allow a crop that does not
                 contain any bbox area. Default to False.
-
         Returns:
             dict: Randomly cropped results, 'img_shape' key in result dict is
                 updated according to crop size.
@@ -1459,22 +1426,27 @@ class MMPad:
         size (tuple, optional): Fixed padding size.
         size_divisor (int, optional): The divisor of padded size.
         pad_to_square (bool): Whether to pad the image into a square.
-           Currently only used for YOLOX. Default: False.
-        pad_val (float, optional): Padding value, 0 by default.
-        seg_pad_val (float, optional): Padding value of segmentation map.
-            Default: 255.
+            Currently only used for YOLOX. Default: False.
+        pad_val (dict, optional): A dict for padding value, the default
+            value is `dict(img=0, masks=0, seg=255)`.
     """
 
     def __init__(self,
                  size=None,
                  size_divisor=None,
                  pad_to_square=False,
-                 pad_val=0,
-                 seg_pad_val=255):
+                 pad_val=dict(img=0, masks=0, seg=255)):
         self.size = size
         self.size_divisor = size_divisor
-        self.pad_val = tuple(pad_val) if isinstance(pad_val, list) else pad_val
-        self.seg_pad_val = seg_pad_val
+        if not isinstance(pad_val, dict):
+            pad_val = tuple(pad_val) if isinstance(pad_val, list) else pad_val
+            warnings.warn(
+                'pad_val of float type is deprecated now, '
+                f'please use pad_val=dict(img={pad_val}, '
+                f'masks={pad_val}, seg=255) instead.', DeprecationWarning)
+            pad_val = dict(img=pad_val, masks=pad_val, seg=255)
+        assert isinstance(pad_val, dict)
+        self.pad_val = pad_val
         self.pad_to_square = pad_to_square
 
         if pad_to_square:
@@ -1488,16 +1460,17 @@ class MMPad:
 
     def _pad_img(self, results):
         """Pad images according to ``self.size``."""
+        pad_val = self.pad_val.get('img', 0)
         for key in results.get('img_fields', ['img']):
             if self.pad_to_square:
                 max_size = max(results[key].shape[:2])
                 self.size = (max_size, max_size)
             if self.size is not None:
                 padded_img = mmcv.impad(
-                    results[key], shape=self.size, pad_val=self.pad_val)
+                    results[key], shape=self.size, pad_val=pad_val)
             elif self.size_divisor is not None:
                 padded_img = mmcv.impad_to_multiple(
-                    results[key], self.size_divisor, pad_val=self.pad_val)
+                    results[key], self.size_divisor, pad_val=pad_val)
             results[key] = padded_img
         results['pad_shape'] = padded_img.shape
         results['pad_fixed_size'] = self.size
@@ -1506,17 +1479,17 @@ class MMPad:
     def _pad_masks(self, results):
         """Pad masks according to ``results['pad_shape']``."""
         pad_shape = results['pad_shape'][:2]
+        pad_val = self.pad_val.get('masks', 0)
         for key in results.get('mask_fields', []):
-            results[key] = results[key].pad(pad_shape, pad_val=self.pad_val)
+            results[key] = results[key].pad(pad_shape, pad_val=pad_val)
 
     def _pad_seg(self, results):
         """Pad semantic segmentation map according to
         ``results['pad_shape']``."""
+        pad_val = self.pad_val.get('seg', 255)
         for key in results.get('seg_fields', []):
             results[key] = mmcv.impad(
-                results[key],
-                shape=results['pad_shape'][:2],
-                pad_val=self.seg_pad_val)
+                results[key], shape=results['pad_shape'][:2], pad_val=pad_val)
 
     def __call__(self, results):
         """Call function to pad images, masks, semantic segmentation maps.
@@ -1542,9 +1515,7 @@ class MMPad:
 @PIPELINES.register_module
 class MMNormalize:
     """Normalize the image.
-
     Added key is "img_norm_cfg".
-
     Args:
         mean (sequence): Mean values of 3 channels.
         std (sequence): Std values of 3 channels.
@@ -1559,10 +1530,8 @@ class MMNormalize:
 
     def __call__(self, results):
         """Call function to normalize images.
-
         Args:
             results (dict): Result dict from loading pipeline.
-
         Returns:
             dict: Normalized results, 'img_norm_cfg' key is added into
                 result dict.
@@ -1646,16 +1615,46 @@ class LoadImageFromFile:
         return repr_str
 
 
+@PIPELINES.register_module()
+class LoadImageFromWebcam(LoadImageFromFile):
+    """Load an image from webcam.
+
+    Similar with :obj:`LoadImageFromFile`, but the image read from webcam is in
+    ``results['img']``.
+    """
+
+    def __call__(self, results):
+        """Call functions to add image meta information.
+
+        Args:
+            results (dict): Result dict with Webcam read image in
+                ``results['img']``.
+
+        Returns:
+            dict: The dict contains loaded image and meta information.
+        """
+
+        img = results['img']
+        if self.to_float32:
+            img = img.astype(np.float32)
+
+        results['filename'] = None
+        results['ori_filename'] = None
+        results['img'] = img
+        results['img_shape'] = img.shape
+        results['ori_shape'] = img.shape
+        results['img_fields'] = ['img']
+        return results
+
+
 @PIPELINES.register_module()
 class LoadMultiChannelImageFromFiles:
     """Load multi-channel images from a list of separate channel files.
-
     Required keys are "img_prefix" and "img_info" (a dict that must contain the
     key "filename", which is expected to be a list of filenames).
     Added or updated keys are "filename", "img", "img_shape",
     "ori_shape" (same as `img_shape`), "pad_shape" (same as `img_shape`),
     "scale_factor" (1.0) and "img_norm_cfg" (means=0 and stds=1).
-
     Args:
         to_float32 (bool): Whether to convert the loaded image to a float32
             numpy array. If set to False, the loaded image is an uint8 array.
@@ -1679,10 +1678,8 @@ class LoadMultiChannelImageFromFiles:
     def __call__(self, results):
         """Call functions to load multiple images and get images meta
         information.
-
         Args:
             results (dict): Result dict from :obj:`mmdet.CustomDataset`.
-
         Returns:
             dict: The dict contains loaded images and meta information.
         """
@@ -1842,7 +1839,7 @@ class LoadAnnotations:
                 If ``self.poly2mask`` is set ``True``, `gt_mask` will contain
                 :obj:`PolygonMasks`. Otherwise, :obj:`BitmapMasks` is used.
         """
-        from mmdet.core import BitmapMasks, PolygonMasks
+        from easycv.utils.mmlab_utils import BitmapMasks, PolygonMasks
 
         h, w = results['img_info']['height'], results['img_info']['width']
         gt_masks = results['ann_info']['masks']
@@ -1909,54 +1906,118 @@ class LoadAnnotations:
 
 
 @PIPELINES.register_module()
-class MMFilterAnnotations:
-    """Filter invalid annotations.
+class LoadPanopticAnnotations(LoadAnnotations):
+    """Load multiple types of panoptic annotations.
+
     Args:
-        min_gt_bbox_wh (tuple[int]): Minimum width and height of ground truth
-            boxes.
-        keep_empty (bool): Whether to return None when it
-            becomes an empty bbox after filtering. Default: True
+        with_bbox (bool): Whether to parse and load the bbox annotation.
+             Default: True.
+        with_label (bool): Whether to parse and load the label annotation.
+            Default: True.
+        with_mask (bool): Whether to parse and load the mask annotation.
+             Default: True.
+        with_seg (bool): Whether to parse and load the semantic segmentation
+            annotation. Default: True.
+        file_client_args (dict): Arguments to instantiate a FileClient.
+            See :class:`mmcv.fileio.FileClient` for details.
+            Defaults to ``dict(backend='disk')``.
     """
 
-    def __init__(self, min_gt_bbox_wh, keep_empty=True):
-        # TODO: add more filter options
-        self.min_gt_bbox_wh = min_gt_bbox_wh
-        self.keep_empty = keep_empty
+    def __init__(self,
+                 with_bbox=True,
+                 with_label=True,
+                 with_mask=True,
+                 with_seg=True,
+                 file_client_args=dict(backend='disk')):
+        if rgb2id is None:
+            raise RuntimeError(
+                'panopticapi is not installed, please install it by: '
+                'pip install git+https://github.com/cocodataset/'
+                'panopticapi.git.')
+
+        super(LoadPanopticAnnotations, self).__init__(
+            with_bbox=with_bbox,
+            with_label=with_label,
+            with_mask=with_mask,
+            with_seg=with_seg,
+            poly2mask=True,
+            # denorm_bbox=False,
+            file_client_args=file_client_args)
+
+    def _load_masks_and_semantic_segs(self, results):
+        """Private function to load mask and semantic segmentation annotations.
+
+        In gt_semantic_seg, the foreground label is from `0` to
+        `num_things - 1`, the background label is from `num_things` to
+        `num_things + num_stuff - 1`, 255 means the ignored label (`VOID`).
+
+        Args:
+            results (dict): Result dict from :obj:`mmdet.CustomDataset`.
+
+        Returns:
+            dict: The dict contains loaded mask and semantic segmentation
+                annotations. `BitmapMasks` is used for mask annotations.
+        """
+        from easycv.utils.mmlab_utils import BitmapMasks
+        if self.file_client is None:
+            self.file_client = mmcv.FileClient(**self.file_client_args)
+
+        filename = osp.join(results['seg_prefix'],
+                            results['ann_info']['seg_map'])
+        img_bytes = self.file_client.get(filename)
+        pan_png = mmcv.imfrombytes(
+            img_bytes, flag='color', channel_order='rgb').squeeze()
+        pan_png = rgb2id(pan_png)
+        gt_masks = []
+        gt_seg = np.zeros_like(pan_png) + 255  # 255 as ignore
+        for mask_info in results['ann_info']['masks']:
+            mask = (pan_png == mask_info['id'])
+            gt_seg = np.where(mask, mask_info['category'], gt_seg)
+            # The legal thing masks
+            if mask_info.get('is_thing'):
+                gt_masks.append(mask.astype(np.uint8))
+
+        if self.with_mask:
+            h, w = results['img_info']['height'], results['img_info']['width']
+            gt_masks = BitmapMasks(gt_masks, h, w)
+            results['gt_masks'] = gt_masks
+            results['mask_fields'].append('gt_masks')
+
+        if self.with_seg:
+            results['gt_semantic_seg'] = gt_seg
+            results['seg_fields'].append('gt_semantic_seg')
+        return results
 
     def __call__(self, results):
-        assert 'gt_bboxes' in results
-        gt_bboxes = results['gt_bboxes']
-        if gt_bboxes.shape[0] == 0:
-            return results
-        w = gt_bboxes[:, 2] - gt_bboxes[:, 0]
-        h = gt_bboxes[:, 3] - gt_bboxes[:, 1]
-        keep = (w > self.min_gt_bbox_wh[0]) & (h > self.min_gt_bbox_wh[1])
-        if not keep.any():
-            if self.keep_empty:
-                return None
-            else:
-                return results
-        else:
-            keys = ('gt_bboxes', 'gt_labels', 'gt_masks', 'gt_semantic_seg')
-            for key in keys:
-                if key in results:
-                    results[key] = results[key][keep]
-            return results
+        """Call function to load multiple types panoptic annotations.
 
-    def __repr__(self):
-        return self.__class__.__name__ + \
-               f'(min_gt_bbox_wh={self.min_gt_bbox_wh},' \
-               f'always_keep={self.always_keep})'
+        Args:
+            results (dict): Result dict from :obj:`mmdet.CustomDataset`.
+
+        Returns:
+            dict: The dict contains loaded bounding box, label, mask and
+                semantic segmentation annotations.
+        """
+
+        if self.with_bbox:
+            results = self._load_bboxes(results)
+            if results is None:
+                return None
+        if self.with_label:
+            results = self._load_labels(results)
+        if self.with_mask or self.with_seg:
+            # The tasks completed by '_load_masks' and '_load_semantic_segs'
+            # in LoadAnnotations are merged to one function.
+            results = self._load_masks_and_semantic_segs(results)
+
+        return results
 
 
 @PIPELINES.register_module()
 class MMMultiScaleFlipAug:
     """Test-time augmentation with multiple scales and flipping.
-
     An example configuration is as followed:
-
     .. code-block::
-
         img_scale=[(1333, 400), (1333, 800)],
         flip=True,
         transforms=[
@@ -1967,12 +2028,9 @@ class MMMultiScaleFlipAug:
             dict(type='ImageToTensor', keys=['img']),
             dict(type='Collect', keys=['img']),
         ]
-
     After MultiScaleFLipAug with above configuration, the results are wrapped
     into lists of the same length as followed:
-
     .. code-block::
-
         dict(
             img=[...],
             img_shape=[...],
@@ -1980,7 +2038,6 @@ class MMMultiScaleFlipAug:
             flip=[False, True, False, True]
             ...
         )
-
     Args:
         transforms (list[dict]): Transforms to apply in each augmentation.
         img_scale (tuple | list[tuple] | None): Images scales for resizing.
@@ -2026,10 +2083,8 @@ class MMMultiScaleFlipAug:
 
     def __call__(self, results):
         """Call function to apply test time augment transforms on results.
-
         Args:
             results (dict): Result dict contains the data to transform.
-
         Returns:
            dict[str: list]: The augmented data, where each value is wrapped
                into a list.
@@ -2061,3 +2116,78 @@ class MMMultiScaleFlipAug:
         repr_str += f'img_scale={self.img_scale}, flip={self.flip}, '
         repr_str += f'flip_direction={self.flip_direction})'
         return repr_str
+
+
+@PIPELINES.register_module()
+class MMFilterAnnotations:
+    """Filter invalid annotations.
+    Args:
+        min_gt_bbox_wh (tuple[float]): Minimum width and height of ground truth
+            boxes. Default: (1., 1.)
+        min_gt_mask_area (int): Minimum foreground area of ground truth masks.
+            Default: 1
+        by_box (bool): Filter instances with bounding boxes not meeting the
+            min_gt_bbox_wh threshold. Default: True
+        by_mask (bool): Filter instances with masks not meeting
+            min_gt_mask_area threshold. Default: False
+        keep_empty (bool): Whether to return None when it
+            becomes an empty bbox after filtering. Default: True
+    """
+
+    def __init__(self,
+                 min_gt_bbox_wh=(1., 1.),
+                 min_gt_mask_area=1,
+                 by_box=True,
+                 by_mask=False,
+                 keep_empty=True):
+        # TODO: add more filter options
+        assert by_box or by_mask
+        self.min_gt_bbox_wh = min_gt_bbox_wh
+        self.min_gt_mask_area = min_gt_mask_area
+        self.by_box = by_box
+        self.by_mask = by_mask
+        self.keep_empty = keep_empty
+
+    def __call__(self, results):
+        if self.by_box:
+            assert 'gt_bboxes' in results
+            gt_bboxes = results['gt_bboxes']
+            instance_num = gt_bboxes.shape[0]
+        if self.by_mask:
+            assert 'gt_masks' in results
+            gt_masks = results['gt_masks']
+            instance_num = len(gt_masks)
+
+        if instance_num == 0:
+            return results
+
+        tests = []
+        if self.by_box:
+            w = gt_bboxes[:, 2] - gt_bboxes[:, 0]
+            h = gt_bboxes[:, 3] - gt_bboxes[:, 1]
+            tests.append((w > self.min_gt_bbox_wh[0])
+                         & (h > self.min_gt_bbox_wh[1]))
+        if self.by_mask:
+            gt_masks = results['gt_masks']
+            tests.append(gt_masks.areas >= self.min_gt_mask_area)
+
+        keep = tests[0]
+        for t in tests[1:]:
+            keep = keep & t
+
+        keys = ('gt_bboxes', 'gt_labels', 'gt_masks')
+        for key in keys:
+            if key in results:
+                results[key] = results[key][keep]
+        if not keep.any():
+            if self.keep_empty:
+                return None
+        return results
+
+    def __repr__(self):
+        return self.__class__.__name__ + \
+               f'(min_gt_bbox_wh={self.min_gt_bbox_wh},' \
+               f'(min_gt_mask_area={self.min_gt_mask_area},' \
+               f'(by_box={self.by_box},' \
+               f'(by_mask={self.by_mask},' \
+               f'always_keep={self.always_keep})'
