@@ -101,7 +101,7 @@ class DistributedMPSampler(_DistributedSampler):
         self.calculate_this_label_list()
         if self.rank == 0:
             print('Before original dataset length is  %d' %
-                  self.dataset.data_source.get_length())
+                  len(self.dataset.data_source))
             print('After  MPRefine dataset length is  %d' % (self.length))
             print('Total %d Label in %s' %
                   (len(self.label_list), type(self.dataset)))
@@ -124,8 +124,8 @@ class DistributedMPSampler(_DistributedSampler):
                                                 this_label_list_size]
             m_per_class = self.dataset.m_per_class
             self.buckets_num = int(
-                int(self.dataset.data_source.get_length() / self.num_replicas)
-                / (m_per_class * this_label_list_size)) + 1
+                int(len(self.dataset.data_source) / self.num_replicas) /
+                (m_per_class * this_label_list_size)) + 1
             self.length = self.buckets_num * m_per_class * int(
                 1 +
                 len(self.label_list) / self.num_replicas)  # self.num_replicas
@@ -136,7 +136,8 @@ class DistributedMPSampler(_DistributedSampler):
 
             # this is a huge bug for split situation
             buckets_num = torch.Tensor([
-                int(self.dataset.data_source.get_length() /
+                int(
+                    len(self.dataset.data_source) /
                     (m_per_class * this_label_list_size))
             ]).to(self.local_rank)
             torch.distributed.all_reduce(buckets_num,
@@ -160,6 +161,7 @@ class DistributedSampler(_DistributedSampler):
         num_replicas=None,
         rank=None,
         shuffle=True,
+        seed=0,
         replace=False,
         split_huge_listfile_byrank=False,
     ):
@@ -170,11 +172,13 @@ class DistributedSampler(_DistributedSampler):
                 distributed training.
             rank (optional): Rank of the current process within num_replicas.
             shuffle (optional): If true (default), sampler will shuffle the indices
+            seed (int, Optional): The seed. Default to 0.
             split_huge_listfile_byrank: if split, return all indice for each rank, because list for each rank has been
                 split before build dataset in dist training
         """
         super().__init__(dataset, num_replicas=num_replicas, rank=rank)
         self.shuffle = shuffle
+        self.seed = seed
         self.replace = replace
         self.unif_sampling_flag = False
         self.split_huge_listfile_byrank = split_huge_listfile_byrank
@@ -196,7 +200,7 @@ class DistributedSampler(_DistributedSampler):
     def generate_new_list(self):
         if self.shuffle:
             g = torch.Generator()
-            g.manual_seed(self.epoch)
+            g.manual_seed(self.epoch + self.seed)
             if self.replace:
                 indices = torch.randint(
                     low=0,
@@ -298,6 +302,7 @@ class DistributedGroupSampler(Sampler):
         Dataset is assumed to be of constant size.
     Args:
         dataset: Dataset used for sampling.
+        seed (int, Optional): The seed. Default to 0.
         num_replicas (optional): Number of processes participating in
             distributed training.
         rank (optional): Rank of the current process within num_replicas.
@@ -306,6 +311,7 @@ class DistributedGroupSampler(Sampler):
     def __init__(self,
                  dataset,
                  samples_per_gpu=1,
+                 seed=0,
                  num_replicas=None,
                  rank=None):
         _rank, _num_replicas = get_dist_info()
@@ -315,6 +321,7 @@ class DistributedGroupSampler(Sampler):
             rank = _rank
         self.dataset = dataset
         self.samples_per_gpu = samples_per_gpu
+        self.seed = seed
         self.num_replicas = num_replicas
         self.rank = rank
         self.epoch = 0
@@ -333,7 +340,7 @@ class DistributedGroupSampler(Sampler):
     def __iter__(self):
         # deterministically shuffle based on epoch
         g = torch.Generator()
-        g.manual_seed(self.epoch)
+        g.manual_seed(self.epoch + self.seed)
 
         indices = []
         for i, size in enumerate(self.group_sizes):
@@ -435,7 +442,6 @@ class DistributedGivenIterationSampler(Sampler):
         self.indices = indices
 
     def gen_new_list(self):
-
         # each process shuffle all list with same seed, and pick one piece according to rank
         np.random.seed(0)
 
