@@ -10,7 +10,7 @@ class RawDataset(BaseDataset):
         super(RawDataset, self).__init__(data_source, pipeline)
 
     def __getitem__(self, idx):
-        results = self.data_source.get_sample(idx)
+        results = self.data_source[idx]
         return self.pipeline(results)
 
     def evaluate(self, scores, keyword, logger=None):
