@@ -2,7 +2,9 @@
 from .activation import FReLU
 from .conv_module import ConvModule, build_conv_layer
 from .conv_ws import ConvWS2d, conv_ws_2d
-from .dist_utils import DistributedLossWrapper, DistributedMinerWrapper
+from .dist_utils import (DistributedLossWrapper, DistributedMinerWrapper,
+                         get_world_size, is_dist_avail_and_initialized,
+                         reduce_mean)
 from .gather_layer import GatherLayer
 from .init_weights import _init_weights, trunc_normal_
 from .multi_pooling import GeMPooling, MultiAvgPooling, MultiPooling
@@ -13,6 +15,9 @@ from .scale import Scale
 # from .weight_init import (bias_init_with_prob, kaiming_init, normal_init,
 #                          uniform_init, xavier_init)
 from .sobel import Sobel
+from .transformer import (MLP, DropPath, Mlp, TransformerEncoder,
+                          TransformerEncoderLayer, _get_activation_fn,
+                          _get_clones)
 
 # __all__ = [
 #    'conv_ws_2d', 'ConvWS2d', 'build_conv_layer', 'ConvModule',
