@@ -9,7 +9,6 @@ import os
 
 import requests
 import torch
-import torch.nn as nn
 try:
     from nni.compression.pytorch import ModelSpeedup
 except ImportError:
@@ -76,7 +75,7 @@ def parse_args():
 def main():
     args = parse_args()
 
-    if args.model_type is not None:
+    if args.model_type is not None and args.config is None:
         assert args.model_type in CONFIG_TEMPLATE_ZOO, 'model_type must be in [%s]' % (
             ', '.join(CONFIG_TEMPLATE_ZOO.keys()))
         print('model_type=%s, config file will be replaced by %s' %
